package com.persagy.service.impl;

import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.quartz.JobDataMap;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.googlecode.aviator.AviatorEvaluator;
import com.googlecode.aviator.Expression;
import com.persagy.cache.AlarmInfoCache;
import com.persagy.cache.AlarmLastTimeCache;
import com.persagy.cache.AlarmRedisCache;
import com.persagy.cache.CreatedAlarmIdsCache;
import com.persagy.client.GroupNettyClient;
import com.persagy.entity.AlarmConditionState;
import com.persagy.entity.AlarmDefine;
import com.persagy.entity.AlarmRecord;
import com.persagy.entity.Condition;
import com.persagy.entity.ExpireAlarmMessage;
import com.persagy.entity.ZktAlarmRecordDO;
import com.persagy.entity.v2.AlarmCondition;
import com.persagy.entity.v2.DmpMessage;
import com.persagy.entity.v2.ItemCodeCondition;
import com.persagy.entity.v2.ObjConditionInfo;
import com.persagy.job.ExpireAlarmQueue;
import com.persagy.mq.RabbitConfig;
import com.persagy.repository.AlarmRecordRepository;
import com.persagy.service.AlarmHandleService;
import com.persagy.service.AlarmQuartzService;
import com.persagy.utils.DateUtils;
import com.persagy.utils.StringUtil;
import com.persagy.utils.ValidateUtils;
import com.persagy.utils.condition.ConditionUtils;
import com.rabbitmq.client.Channel;

import lombok.extern.slf4j.Slf4j;

/**
 * @description:报警处理实现类：包含报警产生和报警恢复
 * @author:LuoGuanhyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/15 001511:14
 * @version:V1.0
 **/
@Service
@Slf4j
public class AlarmHandleServiceImpl implements AlarmHandleService {
	@Autowired
	AlarmInfoCache alarmInfoCache;
	@Autowired
	GroupNettyClient groupNettyClient;
	@Autowired
	AlarmQuartzService alarmQuartzService;
	@Autowired
	AlarmRecordRepository alarmRecordRepository;
	@Autowired
	CreatedAlarmIdsCache createdAlarmIdsCache;
	@Autowired
	AlarmLastTimeCache alarmLastTimeCache;
	@Autowired
	AlarmRedisCache alarmRedisCache;
	@Autowired
	RabbitTemplate rabbitTemplate;

	/**
	 * 集团编码
	 */
	@Value("${group.code}")
	public String groupCode;
	/**
	 * 系统标识，用于创建人
	 */
	@Value("${system.id}")
	public String systemId;
	/**
	 * 处理iot采集值消息线程池
	 */
	private ExecutorService fixedThreadPool = Executors.newFixedThreadPool(30);

	/**
	 * @description: 处理iot采集值消息
	 * @param: msg iot采集值消息
	 * @return: void
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/2/1 3:31 下午
	 * @version: V1.0
	 * @throws IOException
	 */
	@RabbitHandler
	@RabbitListener(queues = "alarm-data-queue")
	public void handleIotMsg(Message message, Channel channel) {
		try {
			JSONArray msgJsonArray = JSON.parseArray(new String(message.getBody()));
			log.debug("接收到采集值：[{}]", msgJsonArray);
			// 校验采集值
			boolean validateIotMsgResult = validateIotMsg(msgJsonArray);
			if (!validateIotMsgResult) {
				log.error("采集值校验未通过");
				return;
			}
			Map<String, JSONArray> map = new HashMap<>();
			for (Object object : msgJsonArray) {
				JSONObject jsonObject = (JSONObject) object;
				// 报警类型
				String itemCode = jsonObject.getString("itemCode");
				// 设备id
				String meterId = jsonObject.getString("meterId");
				String key = meterId + "-" + itemCode;
				JSONArray valueArray = map.get(key);
				if (null != valueArray) {
					valueArray.add(jsonObject);

				} else {
					valueArray = new JSONArray();
					valueArray.add(jsonObject);
				}
				map.put(key, valueArray);
			}

			for (Entry<String, JSONArray> entry : map.entrySet()) {
				fixedThreadPool.execute(() -> {
					for (Object object : entry.getValue()) {
						try {
							JSONObject jsonObject = (JSONObject) object;
							// 采集时间
							String dataTime = jsonObject.getString("dataTime");
							// 报警类型
							String itemCode = jsonObject.getString("itemCode");
							// 设备id
							String meterId = jsonObject.getString("meterId");
							// 信息点id
							String funcId = jsonObject.getString("funcId");
							// 采集值
							String value = jsonObject.getString("value");
							// 处理iot数据
							HashMap<String, Object> paramMap = new HashMap<>();
							JSONArray funcIdArray = JSON.parseArray(funcId);
							JSONArray valuesArray = JSON.parseArray(value);
							for (int i = 0; i < funcIdArray.size(); i++) {
								paramMap.put((String) funcIdArray.getString(i),
										Double.parseDouble(valuesArray.getString(i)));
							}
							triggerOrEndAlarm(dataTime, meterId, itemCode, paramMap);
						} catch (Exception e) {
							log.error("线程池处理发数信息发生错误", e);
						}
					}
					alarmRedisCache.releaseLock("alarm_data_lock:"+entry.getKey());
				});
			}
			channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
		} catch (Exception e) {
			log.error("mq 接收处理发数数据发生错误", e);
		}

	}

//	public static void main(String[] args) throws Exception {
//		String msg = "[{\"dataTime\":\"20210331094500\",\"itemCode\":\"Eqdx003\",\"meterId\":\"Eq1101060001d5e6c3a9a825488e822e56f92a1b37d0\",\"funcId\":\"ACATAH_RunStatus\",\"value\":\"1.0\"}]";
//		AlarmHandleServiceImpl alarmHandleService = new AlarmHandleServiceImpl();
//		Message message = new Message(msg.getBytes(), null);
//		alarmHandleService.handleIotMsg(message, null);
//	}

	/**
	 * 通过报警对象和报警类型找到内存中对应的报警条件，计算报警的产生和恢复
	 *
	 * @param dataTime 采集日期
	 * @param meterId  报警对象id
	 * @param itemCode 报警类型
	 * @param paramMap 采集数据
	 * @author lixing
	 * @version V1.0 2021/10/22 5:19 下午
	 */
	private void triggerOrEndAlarm(String dataTime, String meterId, String itemCode, HashMap<String, Object> paramMap)
			throws Exception {
		// 获取设备的报警条件
		ObjConditionInfo objConditionInfo = alarmRedisCache.getAlarmConditionsByObjId(meterId);
		if (objConditionInfo == null || CollectionUtils.isEmpty(objConditionInfo.getConditions())) {
			log.warn("未获取到设备{}的报警条件", meterId);
			return;
		}
		LinkedList<ItemCodeCondition> itemCodeConditionsByObj = objConditionInfo.getConditions();

		List<ItemCodeCondition> itemCodeConditions = itemCodeConditionsByObj.stream()
				.filter(itemCodeCondition -> itemCode.equals(itemCodeCondition.getItemCode()))
				.collect(Collectors.toList());

		// 遍历这些条件，将采集值带入条件，判断是否产生报警
		for (ItemCodeCondition itemCodeCondition : itemCodeConditions) {
			String conditionId = itemCodeCondition.getConditionId();
//            AlarmCondition alarmCondition = alarmInfoCache.getAlarmCondition(conditionId);
			AlarmCondition alarmCondition = alarmRedisCache.getAlarmCondition(conditionId);

			// 解析触发和恢复条件
			String trigger = alarmCondition.getTriggerBackend();
			Expression triggerExp = AviatorEvaluator.compile(trigger, true);
			String end = alarmCondition.getEndBackend();
			Expression endExp = AviatorEvaluator.compile(end, true);

			// 执行报警条件公式，得到触发和恢复结果
			Boolean triggerResult = null;
			Boolean endResult = null;
			try {
				triggerResult = (Boolean) triggerExp.execute(paramMap);
				endResult = (Boolean) endExp.execute(paramMap);
			} catch (Exception e) {
				log.error("报警条件解析异常");
				log.error("触发条件：");
				log.error(trigger);
				log.error("恢复条件：");
				log.error(end);
				log.error("参数：");
				log.error(paramMap.toString());
			}
			log.debug("triggerResult:[{}],endResult:[{}]", triggerResult, endResult);
			if (triggerResult && endResult) {
				log.error("报警触发条件和报警恢复条件同时满足，请检查，报警定义详情【{}】", alarmCondition);
			}
			AlarmDefine alarmDefine = null;
			try {
				alarmDefine = AlarmDefine.convert2AlarmDefine(itemCodeCondition.getItemCode(), alarmCondition, meterId,
						objConditionInfo.getProjectId());
			} catch (Exception e) {
				log.error("报警条件转换为报警定义出现异常", e);
				return;
			}
			// 报警产生值满足（这里的满足不考虑报警持续时间）
			if (triggerResult) {
				log.debug("满足报警条件");
				// 报警的时候不考虑报警恢复，因为同时报警和报警恢复是不应该出现的
				handleAlarmTriggerData(alarmDefine, dataTime, paramMap);
			} else {
				log.debug("不满足报警条件");
				// 当前数据正常
				handlerNormalData(alarmDefine, dataTime, endResult, paramMap);
			}
		}

	}

	/**
	 * @description: 校验iot消息是否合法
	 * @param: msg iot数据消息
	 * @return: boolean 校验结果
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/2/1 2:26 下午
	 * @version: V1.0
	 */
	private boolean validateIotMsg(JSONArray msgJsonArray) {
		int groupCount = 5;
		for (Object object : msgJsonArray) {
			JSONObject jsonObject = (JSONObject) object;
			if (jsonObject.keySet().size() % groupCount != 0) {
				log.error("采集值有误！每一条iot数据应该包含{}个json 的key", groupCount);
				return false;
			}
			String dataTime = jsonObject.getString("dataTime");
			String meterId = jsonObject.getString("meterId");
			String funcId = jsonObject.getString("funcId");
			String value = jsonObject.getString("value");
			boolean iotDataValidateResult = validateIotData(dataTime, meterId, funcId, value);
			if (!iotDataValidateResult) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @description: 校验iot数据是否合法
	 * @param: dataTime 采集时间
	 * @param: meterId  设备id
	 * @param: funcId   信息点id
	 * @param: value    采集值
	 * @return: boolean 校验结果
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/2/1 3:22 下午
	 * @version: V1.0
	 */
	private boolean validateIotData(String dataTime, String meterId, String funcId, String value) {
		// 校验时间格式
		boolean dateTimeValid = ValidateUtils.validateDateTime(dataTime);
		if (!dateTimeValid) {
			log.error("采集值时间有误，[{}]", dataTime);
			return false;
		}
		// 校验信息点和值是否匹配
		JSONArray funcIdArray = JSON.parseArray(funcId);
		JSONArray valueArray = JSON.parseArray(value);
		if (funcIdArray.size() != valueArray.size()) {
			log.error("信息点个数与值的个数不匹配");
			return false;
		}
		for (Object object : valueArray) {
			try {
				Double.parseDouble(object.toString());
			} catch (NumberFormatException e) {
				log.error("采集值不能被转换为小数，[{}]", object.toString());
				return false;
			}
		}

		return true;
	}

	/**
	 * @param alarmDefine :报警定义
	 * @param dataTime    :IOT采集时间
	 * @param endResult   :报警恢复监测结果
	 * @param paramMap    报警恢复触发值
	 * @description: 当前数据正常判断逻辑
	 * @exception:
	 * @author: LuoGuangyi
	 * @company: Persagy Technology Co.,Ltd
	 * @return: void
	 * @since: 2020/10/21 16:46
	 * @version: V1.0
	 */
	private void handlerNormalData(AlarmDefine alarmDefine, String dataTime, Boolean endResult,
			HashMap<String, Object> paramMap) throws InterruptedException {
		String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
//        AlarmConditionState alarmConditionState = getAlarmDefineStateFromCacheOrDb(defineId);
		AlarmConditionState alarmConditionState = alarmRedisCache.getAlarmDefineState(defineId);

		Condition condition = alarmDefine.getCondition();
		// 当前数据正常，报警状态为正常：清空之前的报警计时，重置回默认状态
		if (AlarmConditionState.State.NORMAL.getType().equals(alarmConditionState.getState())) {
			alarmConditionState = new AlarmConditionState(defineId);
		} else if (AlarmConditionState.State.NOT_DEAL.getType().equals(alarmConditionState.getState())) {
			// 报警恢复条件不满足，不处理
			if (!endResult) {
				return;
			}
			/* 判断是否满足报警恢复持续时间 */
			String alarmSuspendStartTime = alarmConditionState.getAlarmSuspendStartTime();
			if (StringUtil.isEmpty(alarmSuspendStartTime)) {
				alarmSuspendStartTime = dataTime;
				// 设置开始恢复时间
				alarmConditionState.setAlarmSuspendStartTime(alarmSuspendStartTime);
			}
			// 计算报警恢复持续时间
			long alarmSuspendLastTime = DateUtils.betweenTwoTimeSecond(alarmSuspendStartTime, dataTime);
			// 设定的报警恢复持续时间
			int uphold = condition.getEndUphold();
			if (alarmSuspendLastTime < uphold) {
				return;
			}
			/* 超过报警恢复设置的持续时间 */
			log.debug("报警恢复持续时间大于设定时间：[{}]>[{}]", alarmSuspendLastTime, uphold);
			ZktAlarmRecordDO alarmRecordDO = getZktAlarmRecordDOWhenAlarmSuspend(defineId, dataTime, paramMap);
			String alarmId = alarmRecordDO.getAlarmId();
			alarmRedisCache.lock("alarm_record_lock:"+alarmId, alarmId, 10);
			// 发送报警恢复消息
			sendAlarmRecoverMessage(defineId, alarmSuspendStartTime, paramMap, alarmId);
			// 取消过期任务
			cancelRelatedExpireJob(alarmId);
			// 报警恢复，报警状态重置回默认
			alarmConditionState.reset();
		}
		// 更新报警定义缓存
		alarmRedisCache.setAlarmState(defineId, alarmConditionState);
	}

	/**
	 * @description: 发送报警恢复消息
	 * @param: projectId   项目id
	 * @param: recoverTime 恢复时间
	 * @param: paramMap    采集值
	 * @param: alarmId     报警id
	 * @return: com.persagy.entity.NettyMessage<com.persagy.entity.AlarmRecord>
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/2/7 10:03 上午
	 * @version: V1.0
	 */
	private void sendAlarmRecoverMessage(String projectId, String recoverTime, HashMap<String, Object> paramMap,
			String alarmId) {
		// 报警恢复参数
		AlarmRecord alarmRecoverRecord = AlarmRecord.builder().state(2).groupCode(groupCode).projectId(projectId)
				.endTime(DateUtils.parseDate(recoverTime)).endInfo(JSONObject.toJSONString(paramMap)).build();
		alarmRecoverRecord.setId(alarmId);

		DmpMessage dmpMessage = new DmpMessage();
		dmpMessage.setDefaultValue();
		dmpMessage.setMid(alarmId);
		dmpMessage.setProjectId(projectId);
		dmpMessage.setGroupCode(groupCode);
		dmpMessage.setExts(JSONObject.parseObject(JSONObject.toJSONString(alarmRecoverRecord)));
		rabbitTemplate.convertAndSend(RabbitConfig.exchangeAlarm, RabbitConfig.alarmMsgRoutingKey,
				JSON.toJSONString(dmpMessage));
	}

	/**
	 * @description: 取消报警的过期任务
	 * @param: alarmId
	 * @return: void
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/2/7 9:45 上午
	 * @version: V1.0
	 */
	private void cancelRelatedExpireJob(String alarmId) throws InterruptedException {
		ExpireAlarmMessage em = new ExpireAlarmMessage();
		em.setType("2");
		em.setJobName(alarmId);
		em.setJobGroupName("expire");
		ExpireAlarmQueue.getExpireAlarmMessageQueue().produce(em);
	}


	/**
	 * @description: 当报警恢复时拼装边缘端报警信息
	 * @param: defineId 报警定义id
	 * @param: dataTime
	 * @param: paramMap
	 * @return: com.persagy.entity.ZktAlarmRecordDO
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/1/13 下午5:11
	 * @version: V1.0
	 */
	private ZktAlarmRecordDO getZktAlarmRecordDOWhenAlarmSuspend(String defineId, String dataTime,
			HashMap<String, Object> paramMap) {
		if (!alarmRecordRepository.existsById(defineId)) {
			log.error("根据报警定义id[{}]未获取到边缘端报警信息", defineId);
			throw new RuntimeException("根据报警定义id" + defineId + "未获取到边缘端报警信息");
		}
		Optional<ZktAlarmRecordDO> recordOptional = alarmRecordRepository.findById(defineId);
		ZktAlarmRecordDO alarmRecordDO = recordOptional.get();
		alarmRecordDO.setState("2");
		alarmRecordDO.setEndTime(DateUtils.parseDate(dataTime));
		alarmRecordDO.setEndInfo(JSONObject.toJSONString(paramMap));
		return alarmRecordDO;
	}


	/**
	 * @param alarmDefine 报警定义
	 * @param dataTime    :IOT数据采集时间
	 * @param paramMap    报警触发值
	 * @description: 处理当前值报警的情况
	 * @exception:
	 * @author: LuoGuangyi
	 * @company: Persagy Technology Co.,Ltd
	 * @return: void
	 * @since: 2020/10/20 22:31
	 * @version: V1.0
	 */
	private void handleAlarmTriggerData(AlarmDefine alarmDefine, String dataTime, HashMap<String, Object> paramMap)
			throws Exception {
		// 根据defineId获取报警定义状态
		String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
		AlarmConditionState alarmConditionState = alarmRedisCache.getAlarmDefineState(defineId);
		Condition condition = alarmDefine.getCondition();
		// 判断报警是否在有效期内
		boolean inEffectTime = ConditionUtils.inEffectiveTime(condition, dataTime);

		if (inEffectTime) {
			// 已经产生报警
			if (AlarmConditionState.State.NOT_DEAL.getType().equals(alarmConditionState.getState())) {
				// 获取报警持续时间
				String alarmStartTime = alarmConditionState.getAlarmStartTime();
				long lastTime = DateUtils.betweenTwoTimeSecond(alarmStartTime, dataTime);
				String alarmId = alarmRecordRepository.findById(defineId).orElse(new ZktAlarmRecordDO()).getAlarmId();
				if (StringUtils.isNotBlank(alarmId)) {
					// TODO: 2021/11/17 移除alarmLastTimeCache,向mq发送报警持续消息
//                    alarmLastTimeCache.put(alarmId, Integer.valueOf(String.valueOf(lastTime)));
//                    log.debug("开始拼装报警持续消息");
//                    initAlarmContinueMsg(alarmId, lastTime, paramMap, alarmDefine.getProjectId());
					ObjectMapper objectMapper = new ObjectMapper();
					ObjectNode objectNode = objectMapper.createObjectNode();
					objectNode.put("id", alarmId);
					objectNode.put("lastTime", lastTime);
					objectNode.set("iotData", objectMapper.readTree(objectMapper.writeValueAsString(paramMap)));
					objectNode.put("groupCode", groupCode);
					// 推送报警持续消息到MQ
					this.sendToMQ(JSONObject.parseObject(JSONObject.toJSONString(objectNode)), alarmId, alarmDefine.getProjectId());
				}
			}
			// 之前是是正常状态
			if (AlarmConditionState.State.NORMAL.getType().equals(alarmConditionState.getState())) {
				// 报警持续时间
				long alarmLastTime = 0;
				if (StringUtil.isNotEmpty(alarmConditionState.getAlarmStartTime())) {
					alarmLastTime = DateUtils.betweenTwoTimeSecond(alarmConditionState.getAlarmStartTime(), dataTime);
				} else {
					// 设置开始报警时间
					alarmConditionState.setAlarmStartTime(dataTime);
				}

				// 获取报警过期时间
				LocalDateTime expireDateTime = getExpireLocalDateTime(dataTime, condition);
				log.debug("报警持续时间：[{}]", alarmLastTime);
				log.debug("设定的持续时间：[{}]", condition.getTriggerUphold());
				if (alarmLastTime >= condition.getTriggerUphold()) {
					String alarmId = StringUtil.getUUID();
					log.debug("报警持续时间大于设定的持续时间，产生一条报警: [{}]", alarmId);
					// 边缘端保存报警信息
					saveZktAlarmRecordWhenAlarmStart(alarmId, alarmDefine, dataTime);
					// 报警定义状态更新为未处理
					alarmConditionState.setState(AlarmConditionState.State.NOT_DEAL.getType());
					// 发送创建报警记录的消息
					sendCreateAlarmRecordMessage(alarmId, alarmDefine, dataTime, paramMap,
							DateUtils.localDateTime2Date(expireDateTime));

					// 有过期时间，生成报警过期消息
					if (expireDateTime != null) {
						// 过期时间
						log.debug("产生一条定时过期报警消息, 过期时间为：{}", expireDateTime);
						// 创建一条过期任务
						createExpireJob(alarmId, alarmDefine, alarmConditionState, expireDateTime);
					}
				}
			}
		} else {
			// 其他条件全部改成默认值（不报警，不过期，报警开始时间和结束时间为空）
			log.debug("[{}]不在生效时间，重置报警状态", dataTime);
			log.debug("生效时间为：[{}]", condition.getEffectTime());
			alarmConditionState.reset();
		}
		alarmRedisCache.setAlarmState(defineId, alarmConditionState);
	}

	/**
	 * @description: 获取报警过期时间
	 * @param: triggerTime 报警触发时间
	 * @param: condition   报警条件
	 * @return: java.time.LocalDateTime
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/1/8 下午6:16
	 * @version: V1.0
	 */
	private LocalDateTime getExpireLocalDateTime(String triggerTime, Condition condition) {
		// 判断过期设置是否生效
		boolean expireSetEffective = ConditionUtils.expireSetEffective(condition);

		if (!expireSetEffective) {
			return null;
		}

		// 过期时间
		String expireTime = ConditionUtils.getEffectEndTime(condition);
		LocalTime localTime = LocalTime.parse(expireTime, DateTimeFormatter.ofPattern(DateUtils.sdfTimeNotDate));
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DateUtils.sdfDay);
		LocalDate triggerDate = LocalDate.parse(DateUtils.getDate(triggerTime), formatter);
		return LocalDateTime.of(triggerDate, localTime);
	}

	/**
	 * @description: 创建过期任务
	 * @param: alarmId          报警id
	 * @param: alarmDefine      报警定义
	 * @param: alarmDefineState 报警定义状态对象
	 * @param: expireDateTime   过期时间
	 * @return: void
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/1/13 下午3:39
	 * @version: V1.0
	 */
	private void createExpireJob(String alarmId, AlarmDefine alarmDefine, AlarmConditionState alarmConditionState,
			LocalDateTime expireDateTime) throws InterruptedException {
		String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
		ZktAlarmRecordDO alarmRecordDO = ZktAlarmRecordDO.builder().alarmTime(alarmConditionState.getAlarmStartTime())
				.effectEndTime(DateUtils.format(expireDateTime)).definitionId(defineId)
				.itemCode(alarmDefine.getItemCode()).name(alarmDefine.getName()).objId(alarmDefine.getObjId())
				// 要变成已过期
				.state("3").alarmId(alarmId).build();
		JobDataMap jobDataMap = new JobDataMap();
		jobDataMap.put("alarmRecord", alarmRecordDO.toString());
		jobDataMap.put("refire", "0");
		jobDataMap.put("expireTime", DateUtils.format(expireDateTime));
		jobDataMap.put("defineId", defineId);
		// 过期
		jobDataMap.put("state", "3");
		ExpireAlarmMessage em = new ExpireAlarmMessage();
		// 过期消息
		em.setType("1");
		em.setStartTime(DateUtils.localDateTime2Date(expireDateTime));
		em.setJobDataMap(jobDataMap);
		em.setJobName(alarmId);
		em.setJobGroupName("expire");
		ExpireAlarmQueue.getExpireAlarmMessageQueue().produce(em);
	}

	/**
	 * @description: 当报警开始时边缘端保存报警信息
	 * @param: alarmId     报警记录id
	 * @param: alarmDefine 报警定义
	 * @return: com.persagy.entity.ZktAlarmRecordDO
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/1/8 下午5:55
	 * @version: V1.0
	 */
	private ZktAlarmRecordDO saveZktAlarmRecordWhenAlarmStart(String alarmId, AlarmDefine alarmDefine,
			String alarmTime) {
		String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
		synchronized (defineId.intern()) {
			ZktAlarmRecordDO zktAlarmRecordDO = initZktAlarmRecordDO(alarmDefine);
			zktAlarmRecordDO.setAlarmId(alarmId);
			zktAlarmRecordDO.setState("1");
			zktAlarmRecordDO.setAlarmTime(alarmTime);
			alarmRecordRepository.save(zktAlarmRecordDO);
			return zktAlarmRecordDO;
		}
	}

	/**
	 * @description: 初始化边缘端报警记录对象
	 * @param: alarmDefine
	 * @return: com.persagy.entity.ZktAlarmRecordDO
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/1/13 下午5:07
	 * @version: V1.0
	 */
	private ZktAlarmRecordDO initZktAlarmRecordDO(AlarmDefine alarmDefine) {
		String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
		ZktAlarmRecordDO zktAlarmRecordDO = alarmRecordRepository.findById(defineId).orElse(new ZktAlarmRecordDO());
		zktAlarmRecordDO.setDefinitionId(defineId);
		zktAlarmRecordDO.setObjId(alarmDefine.getObjId());
		zktAlarmRecordDO.setItemCode(alarmDefine.getItemCode());
		zktAlarmRecordDO.setProjectId(alarmDefine.getProjectId());
		return zktAlarmRecordDO;
	}

	/**
	 * @description: 发送创建报警记录消息
	 * @param: alarmId     报警id
	 * @param: alarmDefine 报警定义
	 * @param: triggerTime 触发时间
	 * @param: triggerInfo 触发值
	 * @param: expireDate  过期时间
	 * @return: void
	 * @exception:
	 * @author: lixing
	 * @company: Persagy Technology Co.,Ltd
	 * @since: 2021/1/8 下午5:38
	 * @version: V1.0
	 */
	private void sendCreateAlarmRecordMessage(String alarmId, AlarmDefine alarmDefine, String triggerTime,
			HashMap<String, Object> triggerInfo, Date expireDate) throws InterruptedException {
		Condition condition = alarmDefine.getCondition();
		AlarmRecord alarmRecord = AlarmRecord.builder().id(alarmId).category(alarmDefine.getCategory())
				.concern(alarmDefine.getConcern()).level(alarmDefine.getLevel()).projectId(alarmDefine.getProjectId())
				.state(1).triggerTime(DateUtils.parseDate(triggerTime)).remark(alarmDefine.getRemark())
				.triggerInfo(JSONObject.toJSONString(triggerInfo)).condition(condition.toString())
				.effectEndTime(expireDate).groupCode(groupCode).itemCode(alarmDefine.getItemCode())
				.objId(alarmDefine.getObjId()).classCode(alarmDefine.getClassCode()).createUser(systemId).treatState(1)
				.build();
		// TODO: 2021/11/16 消息发送至mq
		this.sendToMQ(JSONObject.parseObject(JSONObject.toJSONString(alarmRecord)), alarmId,
				alarmDefine.getProjectId());
	}

	public void sendToMQ(JSONObject message, String alarmId, String projectId) {
		DmpMessage dmpMessage = new DmpMessage();
		dmpMessage.setDefaultValue();
		dmpMessage.setMid(alarmId);
		dmpMessage.setProjectId(projectId);
		dmpMessage.setGroupCode(groupCode);
		dmpMessage.setExts(message);
		alarmRedisCache.lock("alarm_record_lock:"+alarmId, alarmId, 10);
		rabbitTemplate.convertAndSend(RabbitConfig.exchangeAlarm, RabbitConfig.alarmMsgRoutingKey,
				JSON.toJSONString(dmpMessage));
	}
}
